from wildfly.api.host import HostApiMixin
from .. import base

try:
    from unittest import mock
except ImportError:
    import mock


class HostApiMixinTest(base.BaseTestCase):

	def test_servers(self):
		pass

	def test_server_groups(self):
		pass

	def test_server_operation(self):
		pass

	def test_start_servers(self):
		pass

	def test_stop_servers(self):
		pass

	def test_reload_servers(self):
		pass

	def test_restart_servers(self):
		pass

	def test_read_log_file(self):
		pass



